/*  This file is part of the OpenLB library
 *
 *  Copyright (C) 2008 Orestis Malaspinas, Andrea Parmigiani
 *  E-mail contact: info@openlb.net
 *  The most recent release of OpenLB can be downloaded at
 *  <http://www.openlb.net/>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA  02110-1301, USA.
*/

/** \file
 * Descriptor for all types of 2D and 3D lattices. In principle, thanks
 * to the fact that the OpenLB code is generic, it is sufficient to
 * write a new descriptor when a new type of lattice is to be used.
 *  -- generic code
 */
#ifndef ADVECTION_DIFFUSION_LATTICE_DESCRIPTORS_HH
#define ADVECTION_DIFFUSION_LATTICE_DESCRIPTORS_HH

#include "advectionDiffusionLatticeDescriptors.h"

namespace olb {
namespace descriptors {

// AdvectionDiffusion D2Q5 //////////////////////////////////////////////

template<typename T>
const int D2Q5DescriptorBase<T>::c[D2Q5DescriptorBase<T>::q][D2Q5DescriptorBase<T>::d] = {
  { 0, 0},
  {-1, 0}, {0, -1}, {1,0}, { 0,1}
};

template<typename T>
const int D2Q5DescriptorBase<T>::opposite[D2Q5DescriptorBase<T>::q] = {
  0, 3, 4, 1, 2
};

template<typename T>
const int D2Q5DescriptorBase<T>::vicinity = 1;

template<typename T>
const T D2Q5DescriptorBase<T>::invCs2 = (T)3;

template<typename T>
const T D2Q5DescriptorBase<T>::t[D2Q5DescriptorBase<T>::q] = {
  (T)1-(T)2/invCs2,
  (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2),
  (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2)
};

// AdvectionDiffusion D3Q7 ////////////////////////////////////////////////////

template<typename T>
const int D3Q7DescriptorBase<T>::c[D3Q7DescriptorBase<T>::q][D3Q7DescriptorBase<T>::d] = {
  { 0, 0, 0},

  {-1, 0, 0}, {0,-1, 0},
  { 0, 0,-1}, {1, 0, 0},
  { 0, 1, 0}, {0, 0, 1},
};

template<typename T>
const int D3Q7DescriptorBase<T>::opposite[D3Q7DescriptorBase<T>::q] = {
  0, 4, 5, 6, 1, 2, 3
};

template<typename T>
const int D3Q7DescriptorBase<T>::vicinity = 1;

template<typename T>
const T D3Q7DescriptorBase<T>::invCs2 = (T)4;

template<typename T>
const T D3Q7DescriptorBase<T>::t[D3Q7DescriptorBase<T>::q] = {
  (T)1 -(T)3 / invCs2,

  (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2),
  (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2)
};


// D3Q7 lattice for radiative transport problems @2016 A. Mink et al.
template<typename T>
const int D3Q7DescriptorBaseRTLB<T>::c[D3Q7DescriptorBaseRTLB<T>::q][D3Q7DescriptorBaseRTLB<T>::d] = {
  { 0, 0, 0},

  {-1, 0, 0}, {0,-1, 0},
  { 0, 0,-1}, {1, 0, 0},
  { 0, 1, 0}, {0, 0, 1},
};

template<typename T>
const int D3Q7DescriptorBaseRTLB<T>::opposite[D3Q7DescriptorBaseRTLB<T>::q] = {
  0, 4, 5, 6, 1, 2, 3
};

template<typename T>
const int D3Q7DescriptorBaseRTLB<T>::vicinity = 1;

template<typename T>
const T D3Q7DescriptorBaseRTLB<T>::invCs2 = (T)4;

template<typename T>
const T D3Q7DescriptorBaseRTLB<T>::t[D3Q7DescriptorBaseRTLB<T>::q] = {
  (T)1-(T)3 / invCs2,

  (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2),
  (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2), (T)1/(invCs2*(T)2)
};

///g=0
template<typename T>
const double D3Q7DescriptorBaseRTLB<T>::henyeyPhaseFunction[D3Q7DescriptorBaseRTLB<T>::q][D3Q7DescriptorBaseRTLB<T>::q] = {
  {0, 0, 0, 0, 0, 0, 0},
  {0, 4./3, 4./3., 4./3., 4./3., 4./3., 4./3.},
  {0, 4./3.,  4./3.,  4./3.,  4./3.,  4./3.,  4./3.},
  {0, 4./3.,  4./3.,  4./3.,  4./3.,  4./3.,  4./3.},
  {0, 4./3.,  4./3.,  4./3.,  4./3.,  4./3.,  4./3.},
  {0, 4./3.,  4./3.,  4./3.,  4./3.,  4./3.,  4./3.},
  {0, 4./3.,  4./3.,  4./3.,  4./3.,  4./3.,  4./3.}
};



// RTLB D3Q19 ////////////////////////////////////////////////////

template<typename T>
const int D3Q19DescriptorBaseRTLB<T>::c[D3Q19DescriptorBaseRTLB<T>::q][D3Q19DescriptorBaseRTLB<T>::d] = {
  { 0, 0, 0},

  {-1, 0, 0}, { 0,-1, 0}, { 0, 0,-1},
  {-1,-1, 0}, {-1, 1, 0}, {-1, 0,-1},
  {-1, 0, 1}, { 0,-1,-1}, { 0,-1, 1},

  { 1, 0, 0}, { 0, 1, 0}, { 0, 0, 1},
  { 1, 1, 0}, { 1,-1, 0}, { 1, 0, 1},
  { 1, 0,-1}, { 0, 1, 1}, { 0, 1,-1}
};

template<typename T>
const int D3Q19DescriptorBaseRTLB<T>::opposite[D3Q19DescriptorBaseRTLB<T>::q] = {
  0, 10, 11, 12, 13, 14, 15, 16, 17, 18, 1, 2, 3, 4, 5, 6, 7, 8, 9
};

template<typename T>
const int D3Q19DescriptorBaseRTLB<T>::vicinity = 1;

template<typename T>
const T D3Q19DescriptorBaseRTLB<T>::invCs2 = (T)3;

template<typename T>
const T D3Q19DescriptorBaseRTLB<T>::t[D3Q19DescriptorBaseRTLB<T>::q] = {
  (T)1/(T)3,

  (T)1/(T)18, (T)1/(T)18, (T)1/(T)18,
  (T)1/(T)36, (T)1/(T)36, (T)1/(T)36,
  (T)1/(T)36, (T)1/(T)36, (T)1/(T)36,

  (T)1/(T)18, (T)1/(T)18, (T)1/(T)18,
  (T)1/(T)36, (T)1/(T)36, (T)1/(T)36,
  (T)1/(T)36, (T)1/(T)36, (T)1/(T)36
};


///g=0
template<typename T>
const double D3Q19DescriptorBaseRTLB<T>::henyeyPhaseFunction[D3Q19DescriptorBaseRTLB<T>::q][D3Q19DescriptorBaseRTLB<T>::q] = {
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5},
  {0, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5, 1.5}
};





// D3Q27 ///////////////////////////////////////////////////////////

template<typename T>
const int D3Q27DescriptorBaseRTLB<T>::vicinity = 1;

template<typename T>
const int D3Q27DescriptorBaseRTLB<T>::c
[D3Q27DescriptorBaseRTLB<T>::q][D3Q27DescriptorBaseRTLB<T>::d] = {
  { 0, 0, 0},

  {-1, 0, 0}, { 0,-1, 0}, { 0, 0,-1},
  {-1,-1, 0}, {-1, 1, 0}, {-1, 0,-1},
  {-1, 0, 1}, { 0,-1,-1}, { 0,-1, 1},
  {-1,-1,-1}, {-1,-1, 1}, {-1, 1,-1}, {-1, 1, 1},

  { 1, 0, 0}, { 0, 1, 0}, { 0, 0, 1},
  { 1, 1, 0}, { 1,-1, 0}, { 1, 0, 1},
  { 1, 0,-1}, { 0, 1, 1}, { 0, 1,-1},
  { 1, 1, 1}, { 1, 1,-1}, { 1,-1, 1}, { 1,-1,-1}
};

template<typename T>
const T D3Q27DescriptorBaseRTLB<T>::invCs2 = (T)3;

template<typename T>
const int D3Q27DescriptorBaseRTLB<T>::opposite[D3Q27DescriptorBaseRTLB<T>::q] = {
  0, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26,
  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13
};

template<typename T>
const T D3Q27DescriptorBaseRTLB<T>::t[D3Q27DescriptorBaseRTLB<T>::q] = {
  (T)8/(T)27,

  (T)2/(T)27, (T)2/(T)27, (T)2/(T)27,
  (T)1/(T)54, (T)1/(T)54, (T)1/(T)54,
  (T)1/(T)54, (T)1/(T)54, (T)1/(T)54,
  (T)1/(T)216, (T)1/(T)216, (T)1/(T)216, (T)1/(T)216,

  (T)2/(T)27, (T)2/(T)27, (T)2/(T)27,
  (T)1/(T)54, (T)1/(T)54, (T)1/(T)54,
  (T)1/(T)54, (T)1/(T)54, (T)1/(T)54,
  (T)1/(T)216, (T)1/(T)216, (T)1/(T)216, (T)1/(T)216
};


template<typename T>
const double D3Q27DescriptorBaseRTLB<T>::henyeyPhaseFunction[D3Q27DescriptorBaseRTLB<T>::q][D3Q27DescriptorBaseRTLB<T>::q] = {
  {0., 0., 0., 0., 0., 0., 0., 0., 0. ,0. ,0. ,0., 0., 0., 0., 0., 0., 0., 0., 0.,0. ,0. ,0. ,0., 0. ,0. ,0.},
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  },
  {
    0., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.,
    27./19., 27./19., 27./19., 27./19., 27./19., 27./19., 27./19.
  }
};

}  // namespace descriptors

}  // namespace olb

#endif
